from .healthgraph import create_healthgraph_object


def convert_tcx_to_healthgraph(tcx, notes=None, detect_pauses=False):
    path_points = map(gps_point_to_path_point, tcx.gps_points)
    healthgraph_heartrates = map(
        heartrate_measurement_to_healthgraph_heartrate,
        tcx.heartrate_measurements)

    healthgraph_object = create_healthgraph_object(
        start_time=tcx.start_time,
        duration=tcx.duration,
        activity_type=tcx.activity_type,
        activity_path=path_points,
        secondary_type=tcx.secondary_type,
        heartrates=healthgraph_heartrates,
        equipment=tcx.equipment,
        total_distance=tcx.total_distance,
        notes=notes,
        detect_pauses=detect_pauses)

    return healthgraph_object


def gps_point_to_path_point(tcx_gps_info):
    """
    timestamps are stored as absolute epoch times
    type is ommitted here and it is expected that the correct values will be
    inserted when creating the path list
    """
    return {
        'timestamp': tcx_gps_info.time,
        'latitude': tcx_gps_info.latitude,
        'longitude': tcx_gps_info.longitude,
        'altitude': tcx_gps_info.altitude,
    }


def heartrate_measurement_to_healthgraph_heartrate(tcx_heartrate_info):
    """
    timestamps are stored as absolute epoch times
    """
    return {
        'timestamp': tcx_heartrate_info.time,
        'heart_rate': tcx_heartrate_info.bpm,
    }



from time import (
    gmtime,
    strftime,
)

_RK_TIMESTAMP_FORMAT = '%a, %d %b %Y %H:%M:%S'


def create_healthgraph_object(start_time, duration, activity_type,
                              activity_path=None, secondary_type=None,
                              heartrates=None, equipment=None,
                              total_distance=None, notes=None,
                              detect_pauses=False):
    """
    :param start_time: Activity start time in seconds since epoch
    """
    start_time_str = strftime(_RK_TIMESTAMP_FORMAT, gmtime(start_time))

    healthgraph_object = {
        'type': activity_type,
        'start_time': start_time_str,
        'duration': duration,
        'detect_pauses': detect_pauses,
    }

    if total_distance is not None:
        healthgraph_object['total_distance'] = total_distance

    if activity_path:
        _validate_and_adjust_activity_path(activity_path, start_time)
        healthgraph_object['path'] = activity_path

    if heartrates:
        _adjust_heartrates(heartrates, start_time)
        healthgraph_object['heart_rate'] = heartrates

    if secondary_type is not None:
        healthgraph_object['secondary_type'] = secondary_type

    if equipment is not None:
        healthgraph_object['equipment'] = equipment

    if notes is not None:
        healthgraph_object['notes'] = notes

    return healthgraph_object


def _validate_and_adjust_activity_path(path, activity_start_time):
    if len(path) < 2:
        raise ValueError('The activity path must have at least two points')

    for point in path:
        point['type'] = 'manual'
        _adjust_timestamp(activity_start_time, point)
    path[0]['type'] = 'start'
    path[-1]['type'] = 'end'


def _adjust_heartrates(heartrates, activity_start_time):
    for heartrate in heartrates:
        _adjust_timestamp(activity_start_time, heartrate)


def _adjust_timestamp(activity_start_time, element):
    element['timestamp'] -= activity_start_time



"""
Should output a json file in the format described here:
http://developer.runkeeper.com/healthgraph/fitness-activities#newly-completed-activities
"""
import argh
import json
import logging

from .tcx import get_tcx_activity_from_file
from .conversion import convert_tcx_to_healthgraph

logging.basicConfig()
logger = logging.getLogger(__name__)


@argh.arg('input-file', help='The TCX file to convert')
@argh.arg('output-file', help='The file to output the json file to')
@argh.arg('--notes', help='Notes to attach to this activity')
@argh.arg('--detect-pauses',
          help='If specified then runkeeper should interpret periods of no '
               'entries as pauses')
def main(input_file, output_file, notes=None, detect_pauses=False):
    """
    Converts TCX files to the format expected by the RunKeeper HealthGraph API
    """
    tcx = get_tcx_activity_from_file(input_file)
    healthgraph = convert_tcx_to_healthgraph(
        tcx, notes=notes, detect_pauses=detect_pauses)
    with open(output_file, 'w') as out_buffer:
        json.dump(healthgraph, out_buffer)



import copy
import logging
import time
import xmltodict

from itertools import chain

_TCX_TIME_FORMAT = '%Y-%m-%dT%H:%M:%SZ'

logger = logging.getLogger(__name__)


class TrackpointError(ValueError):

    def __init__(self, message, tcx_element=None):
        Exception.__init__(self, message)
        self.tcx_element = tcx_element


class Activity(object):

    def __init__(self, activity_dict):
        self._activity_dict = activity_dict

        self._activity_type = activity_dict['@Sport']

        self._laps = activity_dict['Lap']
        if not isinstance(self._laps, list):
            # If there is only one lap in the activity it is not in a list. We
            # need to wrap it up in a list in that case
            self._laps = [self._laps]

        self._trackpoints = self._get_activity_trackpoints(self._laps)

        self._total_distance = sum(map(self._get_lap_distance, self._laps))
        self._duration = sum(map(self._get_lap_duration, self._laps))

    @property
    def activity_type(self):
        return self._activity_type

    @property
    def total_distance(self):
        return self._total_distance

    @property
    def secondary_type(self):
        #TODO: If this has an entry in tcx files then lets check for it
        return None
    @property
    def equipment(self):
        #TODO: If this has an entry in tcx files then lets check for it
        return None

    @property
    def start_time(self):
        return tcx_timestamp_to_epoch(self._laps[0]['@StartTime'])

    @property
    def duration(self):
        return self._duration

    @property
    def gps_points(self):
        return tuple(
            trackpoint.gps_info for trackpoint in self._trackpoints
            if trackpoint.gps_info is not None)

    @property
    def heartrate_measurements(self):
        return tuple(
            trackpoint.heartrate_info for trackpoint in self._trackpoints
            if trackpoint.heartrate_info is not None)

    @classmethod
    def _get_activity_trackpoints(cls, laps):
        trackpoint_dicts = chain.from_iterable(
            map(cls._get_lap_trackpoints, laps))
        return tuple(Trackpoint(tp) for tp in trackpoint_dicts)

    @staticmethod
    def _get_lap_trackpoints(lap):
        if 'Track' not in lap:
            return []

        trackpoints = lap['Track']['Trackpoint']
        return trackpoints if isinstance(trackpoints, list) else [trackpoints]

    @staticmethod
    def _get_lap_distance(lap):
        return float(lap['DistanceMeters'])

    @staticmethod
    def _get_lap_duration(lap):
        return float(lap['TotalTimeSeconds'])


class _TrackpointBase(object):

    def __init__(self, trackpoint_dict):
        try:
            self._time = tcx_timestamp_to_epoch(trackpoint_dict['Time'])
        except KeyError as e:
            raise TrackpointError(
                'Could not extract %s from trackpoint' % e, trackpoint_dict)

    @property
    def time(self):
        """
        Returns trackpoint time in number of seconds since epoch
        """
        return self._time

    @property
    def trackpoint_dict(self):
        return copy.deepcopy(self._trackpoint_dict)


class Trackpoint(_TrackpointBase):
    """
    Immutable object to give access to trackpoint data
    """

    def __init__(self, trackpoint_dict):
        super(Trackpoint, self).__init__(trackpoint_dict)

        try:
            self._gps_info = GPSInfo(trackpoint_dict)
        except TrackpointError as e:
            logger.info('gps info skipped. %s : %s', e, e.tcx_element)
            self._gps_info = None

        try:
            self._heartrate_info = HeartrateInfo(trackpoint_dict)
        except TrackpointError as e:
            logger.debug('heartrate info skipped. %s : %s', e, e.tcx_element)
            self._heartrate_info = None

    @property
    def gps_info(self):
        return self._gps_info

    @property
    def heartrate_info(self):
        return self._heartrate_info


class GPSInfo(_TrackpointBase):

    def __init__(self, trackpoint_dict):
        super(GPSInfo, self).__init__(trackpoint_dict)
        try:
            self._altitude = float(trackpoint_dict['AltitudeMeters'])
            self._longitude = float(
                trackpoint_dict['Position']['LongitudeDegrees'])
            self._latitude = float(
                trackpoint_dict['Position']['LatitudeDegrees'])
        except KeyError as e:
            raise TrackpointError(
                'Could not extract %s from trackpoint' % e, trackpoint_dict)

    @property
    def latitude(self):
        return self._latitude

    @property
    def longitude(self):
        return self._longitude

    @property
    def altitude(self):
        return self._altitude


class HeartrateInfo(_TrackpointBase):

    def __init__(self, trackpoint_dict):
        super(HeartrateInfo, self).__init__(trackpoint_dict)
        try:
            self._bpm = int(trackpoint_dict['HeartRateBpm']['Value'])
        except KeyError as e:
            raise TrackpointError(
                'Could not extract %s from trackpoint' % e, trackpoint_dict)

    @property
    def bpm(self):
        return self._bpm


def get_tcx_activity_from_file(filepath):
    return get_activity(read_tcx_file(filepath))


def get_activity(tcx):
    return Activity(tcx['TrainingCenterDatabase']['Activities']['Activity'])


def read_tcx_file(filepath):
    """
    Reads in a tcx file and returns a dict
    """
    return xmltodict.parse(open(filepath, 'r'))


def tcx_timestamp_to_epoch(time_str):
    time_tuple = time.strptime(time_str, _TCX_TIME_FORMAT)
    return time.mktime(time_tuple)



#!/usr/bin/env python

import os
import sys

# Sets __version__
try:
    # Works with Python 2.x
    execfile('tcx2rk/_version.py')
except NameError:
    # Works with Python 3.x
    exec(open('tcx2rk/_version.py').read())

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()

try:
    import pypandoc
    description = pypandoc.convert('README.markdown', 'rst')
except (IOError, ImportError):
    description = ''

requires = [
    'argh==0.24.1',
    'argcomplete==0.8.1',
    'xmltodict==0.8.6',
]

setup(
    name='tcx2rk',
    version=__version__,
    description='Convert TCX files to the Runkeeper format',
    long_description=description,
    author='Brendan Maguire',
    author_email='maguire.brendan@gmail.com',
    install_requires=requires,
    license='Apache 2.0',
    scripts=['bin/tcx2rk'],
    url='https://github.com/brendanmaguire/tcx2rk',
    packages=['tcx2rk'],
)
